<!--- Initialize a variable called Continue to control the transaction --->
<CFSET Continue = "Yes">

<!--- begin transaction --->
<CFTRANSACTION ACTION="BEGIN">

<!--- wrap the INSERT INTO in a CFTRY block --->
<CFTRY>
<!--- Select the name, title, email, and phone ext for each employee in
      the EmployeeDirectory table that belongs to the Sales department 
      and use it to populate a new table called SalesDirectory --->
<CFQUERY NAME="MakeSalesDirectory" DATASOURCE="ProgrammingCF">
         SELECT Name, Title, Email, PhoneExt
         INTO SalesDirectory
         FROM EmployeeDirectory
         WHERE Department = 'Sales'
</CFQUERY>

<!--- if a database error occurs, rollback the transaction and set the
      Continue variable to No. --->
<CFCATCH TYPE="Database">
    <CFTRANSACTION ACTION="ROLLBACK"/>
    <CFSET ProblemQuery = "MakeSalesDirectory">
    <CFSET Continue = "No">
</CFCATCH>
</CFTRY>

<!--- if the INSERT INTO was successful, commit the transaction and
      execute another query to insert a new record into the table we
      created with the last query. --->
<CFIF Continue EQ "Yes">
<CFTRY>
<CFTRANSACTION ACTION="COMMIT"/>
<CFQUERY NAME="InsertRecord" DATASOURCE="ProgrammingCF">
         INSERT INTO SalesDirectory(Name, Title, Email, PhoneExt)
         VALUES('Lynda Newton', 'Account Manager', 'lynda@example.com', 1261)
</CFQUERY>         
 
<!--- if a database error occurs, rollback the transaction and set the
      Continue variable to No. --->
<CFCATCH TYPE="DATABASE">
    <CFTRANSACTION ACTION="ROLLBACK"/>
    <CFSET ProblemQuery = "InsertRecord">
    <CFSET Continue = "No">
</CFCATCH>
</CFTRY>
</CFIF>

<!--- if the record was successfully added, commit the transaction --->
<CFIF Continue EQ "Yes">
    <CFTRANSACTION ACTION="COMMIT"/>
</CFIF>
</CFTRANSACTION> 

<!--- if both transactions were successful, generate a table containing all
      of the records from the new table.  If not display a message letting
      the user know there was a problem. --->
<CFIF Continue EQ "Yes">

<!--- retrieve all of the records from the Sales Directory table we just 
      created --->
<CFQUERY NAME="GetEmployees" DATASOURCE="ProgrammingCF">
         SELECT Name, Title, Email, PhoneExt
         FROM SalesDirectory
         ORDER BY Name
</CFQUERY>

<H2>All queries in the transaction executed successfully</H2>
<H3>Below is the data from the new table:</H3>

<TABLE CELLPADDING="3" CELLSPACING="0">
<TR BGCOLOR="#888888">
  <TH>Name</TH>
  <TH>Title</TH>
  <TH>E-mail</TH>
  <TH>Phone Ext.</TH>
</TR>

<!--- dynamically generate a table containing all of the records returned 
      by the query --->
<CFOUTPUT QUERY="GetEmployees">
<TR BGCOLOR="##C0C0C0">
  <TD>#Name#</TD>
  <TD>#Title#</TD>
  <TD>#Email#</TD>
  <TD>#PhoneExt#</TD>
</TR>
</CFOUTPUT>
</TABLE>

<CFELSE>
<CFOUTPUT>
<H2>An Error has occurred.  All queries have been rolled back</H2>
<B>The query that caused the error is: <I>#ProblemQuery#</I>.</B>
</CFOUTPUT>
</CFIF>
