<CFSETTING ENABLECFOUTPUTONLY="Yes">
<!---------------------------------------------------------------------------
NAME:           CF_RandomPassword
FILE:           RandomPassword.cfm
VERSION:        1.2       
CREATED:        02/23/1998
LAST MODIFIED:  06/30/2000
AUTHOR:         Rob Brooks-Bilson (rbils@amkor.com)
DESCRIPTION:    CF_RandomPassword is a custom CFML tag that allows a user
                to generate random passwords for use in their applications.
                The parameters governing the creation of the password are
                fully customizable.  Passwords can be of any length and
                can contain numbers, symbols, uppercase, and lowercase
                letters in any combination.  This tag can easily be
                incorporated into registration modules to allow for the
                automatic generation of random passwords.
ATTRIBUTES:     REQUIRED
                OutVar = name to give the structure that is returned as a
                         caller var
                OPTIONAL
                NumberOfCharacters(numeric) = The lengh in characters of the
                                              password
                UseSymbols(Yes/No) = Indicates whether to include symbols in
                                     the password                           
                UseNumbers(Yes/No) = Indicates whether to include numbers in
                                     the password
                UseUppercaseLetters(Yes/No) = Indicates whether to include 
                                              upper-case letters in the
                                              password
                UseLowercaseLetters(Yes/No) = Indicates whether to include 
                                              lower-case letters in the 
                                              password
KNOWN ISSUES: None
---------------------------------------------------------------------------->

<!--- Check to see if OutVar was passed in.  If so, check to make
      sure that it is not a blank value.  If any of these
      checks return False, throw an exception. --->
<CFIF IsDefined('Attributes.OutVar') AND Attributes.OutVar NEQ "">
    <CFSET OutVar = Attributes.OutVar>
<CFELSE>
    <CFTHROW MESSAGE="You failed to pass a required attribute: <B>OutVar</B>"
             TYPE="Custom.Tag.RandomPassword">
</CFIF>

<!--- Check to see if NumberOfCharacters was passed in.  If so, check to make
      sure it is numeric.  If any of these checks return False, set a default 
      length of 7 for the password.  --->
<CFIF IsDefined('Attributes.NumberOfCharacters') AND 
                 IsNumeric(Attributes.NumberOfCharacters)>
    <CFSET NumberOfCharacters = Int(Attributes.NumberOfCharacters)>
<CFELSE>
    <CFSET NumberOfCharacters = 7>
</CFIF>

<!--- Check to see if UseSymbols was passed in.  If so, check to make
      sure it is a Boolean.  If any of these checks return False, default 
      UseSymbols to No. --->
<CFIF IsDefined('Attributes.UseSymbols') AND IsBoolean(Attributes.UseSymbols)>
    <CFSET UseSymbols=attributes.UseSymbols>
<CFELSE>
    <CFSET UseSymbols="No">
</CFIF>

<!--- Check to see if UseNumbers was passed in.  If so, check to make
      sure it is a Boolean.  If any of these checks return False, default 
      UseNumbers to Yes. --->
<CFIF IsDefined('Attributes.UseNumbers') AND IsBoolean(Attributes.UseNumbers)>
    <CFSET UseNumbers=Attributes.UseNumbers>
<CFELSE>
    <CFSET UseNumbers="Yes">
</CFIF>      

<!--- Check to see if UseUppercaseLetters was passed in.  If so, check to make
      sure it is a Boolean.  If any of these checks return False, default 
      UseUppercaseLetters to Yes. --->
<CFIF IsDefined('Attributes.UseUppercaseLetters') AND 
      IsBoolean(Attributes.UseUppercaseLetters)>
    <CFSET UseUppercaseLetters = Attributes.UseUppercaseLetters>
<CFELSE>
    <CFSET UseUppercaseLetters="Yes">
</CFIF> 

<!--- Check to see if UseLowercaseLetters was passed in.  If so, check to make
      sure it is a Boolean.  If any of these checks return False, default 
      UseLowercaseLetters to Yes. --->
<CFIF IsDefined('Attributes.UseLowercaseLetters') AND
                 IsBoolean(Attributes.UseLowercaseLetters)>
    <CFSET UseLowercaseLetters = Attributes.UseLowercaseLetters>
<CFELSE>
    <CFSET UseLowercaseLetters="Yes">
</CFIF>    

<!--- make sure all attributes are NOT set to No.  If they are, send back an 
      error message and abort processing.  This is an example of a required
      attribute. --->
<CFIF UseSymbols IS "No" AND UseNumbers IS "No" AND UseUpperCaseLetters IS "No" 
      AND UseLowerCaseLetters IS "No">
     <CFTHROW MESSAGE="No random password can be generated because all 
                       attributes have a <B>""No""</B> Value."
              TYPE="Custom.Tag.RandomPassword">
<CFELSE>

<!--- initialize variables. CharacterList holds the ASCII codes for each character 
      and CharacterString holds the actual characters after they are generated --->
<CFSET CharacterList = "">
<CFSET CharacterString = "">

<!--- This next section builds the total available character list with values
      based on the parameters passed to the tag.  Each character is determined  
      by its ASCII character code.  If you don't want to use certain characters 
      (like the lowercase l and the number 1 and the <, ", and > characters), 
      simply remove their corresponding ASCII character code from the 
      appropriate list.  I'll leave that up to you. --->
<CFIF UseSymbols>
  <CFSET Symbols="33,34,35,36,37,38,39,40,41,42,43,44,45,46,47,58,59,60,61,62,63,
                  64,91,92,93,94,95,96,123,124,125,126">
  <CFSET CharacterList=ListAppend(CharacterList, Symbols, ",")>
</CFIF>

<CFIF UseNumbers>
  <CFSET Numbers="48,49,50,51,52,53,54,55,56,57">
  <CFSET CharacterList=ListAppend(CharacterList, Numbers, ",")>
</CFIF>

<CFIF UseUppercaseLetters>
  <CFSET UppercaseLetters="65,66,67,68,69,70,71,72,73,74,75,76,77,78,79,80,81,82,
                           83,84,85,86,87,88,89,90">
  <CFSET CharacterList=ListAppend(CharacterList, UppercaseLetters, ",")>
</CFIF>

<CFIF UseLowercaseLetters>
  <CFSET LowercaseLetters="97,98,99,100,101,102,103,104,105,106,107,108,109,110,
                           111,112,113,114,115,116,117,118,119,120,121,122">
  <CFSET CharacterList=ListAppend(CharacterList, LowercaseLetters, ",")>
</CFIF>

<!--- get the length of the character list --->
<CFSET TheLength=ListLen(CharacterList)>
    
<!--- Create a loop that iterates the same number of times as the length of the
      password.  For each iteration of the loop, one character is randomly
      chosen from the character list and added to the character string.  When
      the loop has finished iterating, you have a finished password  --->
<CFLOOP INDEX="CharacterPlaces" FROM="1" TO="#NumberOfCharacters#">

<CFSET GetPosition=RandRange(1,TheLength)>
<CFSET Character=ListGetAt(CharacterList, GetPosition, ",")>
<CFSET CharacterString=ListAppend(CharacterString, CHR(Character), ",")>
</CFLOOP>

<!--- return the random password back to the calling template with the 
      caller variable name specified by the OutVar value --->
<CFSET "caller.#OutVar#" = ListChangeDelims(CharacterString, "")>
</CFIF> 
<CFSETTING ENABLECFOUTPUTONLY="No">
